@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013 Ofer Kashayov - oferkv@live.com
+ *  Copyright (C) 2013-2014 Ofer Kashayov - oferkv@live.com
  *  This file is part of Phototonic Image Viewer.
  *
  *  Phototonic is free software: you can redistribute it and/or modify
@@ -18,18 +18,19 @@
 
 #include <QGraphicsDropShadowEffect>
 #include <math.h>
+#include <exiv2/exiv2.hpp>
 #include "global.h"
 #include "imageview.h"
-#include "thumbview.h"
 
 #define CLIPBOARD_IMAGE_NAME		"clipboard.png"
 #define ROUND(x) ((int) ((x) + 0.5))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 
-ImageView::ImageView(QWidget *parent) : QWidget(parent)
+ImageView::ImageView(QWidget *parent, MetadataCache *mdCache) : QWidget(parent)
 {
-	mainWindow = parent;
+	this->mainWindow = parent;
+	this->mdCache = mdCache;
 	cursorIsHidden = false;
 	moveImageLocked = false;
 	mirrorLayout = LayNone;
@@ -37,13 +38,12 @@ ImageView::ImageView(QWidget *parent) : QWidget(parent)
 	imageLabel->setScaledContents(true);
 	isAnimation = false;
 	anim = 0;
-
 	setPalette(QPalette(GData::backgroundColor));
 
-	QHBoxLayout *mainHLayout = new QHBoxLayout();
-	mainHLayout->setContentsMargins(0, 0, 0, 0);
-	mainHLayout->setSpacing(0);
-	mainHLayout->addWidget(imageLabel);
+	QHBoxLayout *mainLayout = new QHBoxLayout();
+	mainLayout->setContentsMargins(0, 0, 0, 0);
+	mainLayout->setSpacing(0);
+	mainLayout->addWidget(imageLabel);
 
 	scrlArea = new QScrollArea;
 	scrlArea->setContentsMargins(0, 0, 0, 0);
@@ -53,16 +53,17 @@ ImageView::ImageView(QWidget *parent) : QWidget(parent)
 	scrlArea->verticalScrollBar()->blockSignals(true);
 	scrlArea->horizontalScrollBar()->blockSignals(true);
 	scrlArea->setFrameStyle(0);
-	scrlArea->setLayout(mainHLayout);
+	scrlArea->setLayout(mainLayout);
 	scrlArea->setWidgetResizable(true);
 
-	QVBoxLayout *mainVLayout = new QVBoxLayout;
-	mainVLayout->setContentsMargins(0, 0, 0, 0);
-	mainVLayout->addWidget(scrlArea);
-	this->setLayout(mainVLayout);
+	QVBoxLayout *scrollLayout = new QVBoxLayout;
+	scrollLayout->setContentsMargins(0, 0, 0, 0);
+	scrollLayout->setSpacing(0);
+	scrollLayout->addWidget(scrlArea);
+	this->setLayout(scrollLayout);
 
 	infoLabel = new QLabel(this);
-	infoLabel->setVisible(false);
+	infoLabel->setVisible(GData::enableImageInfoFS);
 	infoLabel->setMargin(3);
 	infoLabel->move(10, 10);
 	infoLabel->setStyleSheet("QLabel { background-color : black; color : white; border-radius: 3px} ");
@@ -82,27 +83,23 @@ ImageView::ImageView(QWidget *parent) : QWidget(parent)
 	mouseMovementTimer = new QTimer(this);
 	connect(mouseMovementTimer, SIGNAL(timeout()), this, SLOT(monitorCursorState()));
 
-	GData::cropLeft = 0;
-	GData::cropTop = 0;
-	GData::cropWidth = 0;
-	GData::cropHeight = 0;
+	GData::cropLeft = GData::cropTop = GData::cropWidth = GData::cropHeight = 0;
+	GData::cropLeftPercent= GData::cropTopPercent = GData::cropWidthPercent = GData::cropHeightPercent = 0;
 
 	GData::hueVal = 0;
 	GData::saturationVal = 100;
 	GData::lightnessVal = 100;
-	GData::hueSatEnabled = false;
 	GData::hueRedChannel = true;
 	GData::hueGreenChannel = true;
 	GData::hueBlueChannel = true;
 
-	GData::brightContrastEnabled = false;
-	GData::contrastVal = CONTRAST_MID;
-	GData::brightVal = BRIGHTNESS_MID;
+	GData::contrastVal = 78;
+	GData::brightVal = 100;
 
-	GData::dialogLastX = 0;
-	GData::dialogLastY = 0;
+	GData::dialogLastX = GData::dialogLastY = 0;
 
 	newImage = false;
+	cropBand = 0;
 }
 
 static unsigned int getHeightByWidth(int imgWidth, int imgHeight, int newWidth)
@@ -127,94 +124,118 @@ static inline int calcZoom(int size)
 void ImageView::resizeImage()
 {
 	static bool busy = false;
-	if (busy || (!imageLabel->pixmap() && !anim))
+	if (busy || (!imageLabel->pixmap() && !anim)) {
 		return;
+	}
 	busy = true;
-
-	imageLabel->setVisible(false);
+	
+	int imageViewWidth = this->size().width();
+	int imageViewHeight = this->size().height();
 	QSize imgSize = isAnimation? anim->currentPixmap().size() : imageLabel->pixmap()->size();
 
-	if (tempDisableResize)
+	if (tempDisableResize) {
 		imgSize.scale(calcZoom(imgSize.width()), calcZoom(imgSize.height()), Qt::KeepAspectRatio);
-	else
-	{
-		switch(GData::zoomInFlags)
-		{
+	} else {
+		switch(GData::zoomInFlags) {
 			case Disable:
-				if (imgSize.width() < size().width() && imgSize.height() < size().height())
-					imgSize.scale(calcZoom(imgSize.width()), calcZoom(imgSize.height()),
-																					Qt::KeepAspectRatio);
+				if (imgSize.width() < imageViewWidth && imgSize.height() < imageViewHeight) {
+					imgSize.scale(	calcZoom(imgSize.width()),
+									calcZoom(imgSize.height()),
+									Qt::KeepAspectRatio);
+				}
 				break;
 				
 			case WidthNHeight:
-				if (imgSize.width() < size().width() && imgSize.height() < size().height())
-					imgSize.scale(calcZoom(size().width()), calcZoom(size().height()),
-																					Qt::KeepAspectRatio);
+				if (imgSize.width() < imageViewWidth && imgSize.height() < imageViewHeight) {
+					imgSize.scale(	calcZoom(imageViewWidth),
+									calcZoom(imageViewHeight),
+									Qt::KeepAspectRatio);
+				}
 				break;
 
 			case Width:
-				if (imgSize.width() < size().width())
-					imgSize.scale(calcZoom(size().width()), 
-						calcZoom(getHeightByWidth(imgSize.width(), imgSize.height(), size().width())),
-						Qt::KeepAspectRatio);
+				if (imgSize.width() < imageViewWidth) {
+					imgSize.scale(	calcZoom(imageViewWidth), 
+									calcZoom(getHeightByWidth(	imgSize.width(),
+																imgSize.height(),
+																imageViewWidth)),
+									Qt::KeepAspectRatio);
+				}
 				break;
 				
 			case Height:
-				if (imgSize.height() < size().height())
-					imgSize.scale(calcZoom(getWidthByHeight(imgSize.height(), imgSize.width(),
-									size().height())), calcZoom(size().height()), Qt::KeepAspectRatio);
+				if (imgSize.height() < imageViewHeight) {
+					imgSize.scale(	calcZoom(getWidthByHeight(	imgSize.height(), 
+																imgSize.width(),
+																imageViewHeight)),
+									calcZoom(imageViewHeight),
+									Qt::KeepAspectRatio);
+				}
 				break;
 
 			case Disprop:
 				int newWidth = imgSize.width(), newHeight = imgSize.height();
-				if (newWidth < size().width())
-					newWidth = size().width();
-				if (newHeight < size().height())
-					newHeight = size().height();
+				if (newWidth < imageViewWidth) {
+					newWidth = imageViewWidth;
+				}
+				if (newHeight < imageViewHeight) {
+					newHeight = imageViewHeight;
+				}
 				imgSize.scale(calcZoom(newWidth), calcZoom(newHeight), Qt::IgnoreAspectRatio);
 				break;
 		}
 
-		switch(GData::zoomOutFlags)
-		{
+		switch(GData::zoomOutFlags) {
 			case Disable:
-				if (imgSize.width() >= size().width() || imgSize.height() >= size().height())
-					imgSize.scale(calcZoom(imgSize.width()), calcZoom(imgSize.height()),
-																					Qt::KeepAspectRatio);
+				if (imgSize.width() >= imageViewWidth || imgSize.height() >= imageViewHeight) {
+					imgSize.scale(	calcZoom(imgSize.width()), 
+									calcZoom(imgSize.height()),
+									Qt::KeepAspectRatio);
+				}
 				break;
 		
 			case WidthNHeight:
-				if (imgSize.width() >= size().width() || imgSize.height() >= size().height())
-					imgSize.scale(calcZoom(size().width()), calcZoom(size().height()),
-																					Qt::KeepAspectRatio);
+				if (imgSize.width() >= imageViewWidth || imgSize.height() >= imageViewHeight) {
+					imgSize.scale(	calcZoom(imageViewWidth),
+									calcZoom(imageViewHeight),
+									Qt::KeepAspectRatio);
+				}
 				break;
 
 			case Width:
-				if (imgSize.width() > size().width())
-					imgSize.scale(calcZoom(size().width()), 
-						calcZoom(getHeightByWidth(imgSize.width(), imgSize.height(), size().width())),
-																					Qt::KeepAspectRatio);
+				if (imgSize.width() > imageViewWidth) {
+					imgSize.scale(	calcZoom(imageViewWidth), 
+									calcZoom(getHeightByWidth(	imgSize.width(), 
+																imgSize.height(), 
+																imageViewWidth)),
+									Qt::KeepAspectRatio);
+				}
 				break;
 				
 			case Height:
-				if (imgSize.height() > size().height())
-					imgSize.scale(calcZoom(getWidthByHeight(imgSize.height(), imgSize.width(), 
-																					size().height())),
-						calcZoom(size().height()), Qt::KeepAspectRatio);
-			
+				if (imgSize.height() > imageViewHeight) {
+					imgSize.scale(	calcZoom(getWidthByHeight(	imgSize.height(),
+																imgSize.width(), 
+																imageViewHeight)),
+									calcZoom(imageViewHeight),
+									Qt::KeepAspectRatio);
+				}
 				break;
 
 			case Disprop:
 				int newWidth = imgSize.width(), newHeight = imgSize.height();
-				if (newWidth > size().width())
-					newWidth = size().width();
-				if (newHeight > size().height())
-					newHeight = size().height();
+				if (newWidth > imageViewWidth) {
+					newWidth = imageViewWidth;
+				}
+				if (newHeight > imageViewHeight) {
+					newHeight = imageViewHeight;
+				}
 				imgSize.scale(calcZoom(newWidth), calcZoom(newHeight), Qt::IgnoreAspectRatio);
 				break;
 		}
 	}
 
+	imageLabel->setVisible(false);
 	imageLabel->setFixedSize(imgSize);
 	imageLabel->setVisible(true);
 	centerImage(imgSize);
@@ -235,48 +256,20 @@ void ImageView::showEvent(QShowEvent *event)
 
 void ImageView::centerImage(QSize &imgSize)
 {
-	int newX = imageLabel->pos().x();
-	int newY = imageLabel->pos().y();
+	int newX = (this->size().width() - imgSize.width()) / 2;
+	int newY = (this->size().height() - imgSize.height()) / 2;
 
-	newX = (size().width() - imgSize.width()) / 2;
-	newY = (size().height() - imgSize.height()) / 2;
-
-	if (newX != imageLabel->pos().x() || newY != imageLabel->pos().y())
+	if (newX != imageLabel->pos().x() || newY != imageLabel->pos().y()) {
 		imageLabel->move(newX, newY);
+	}
 }
 
-void ImageView::rotateByExifRotation(QImage &image, const QString &imageFullPath)
+void ImageView::rotateByExifRotation(QImage &image, QString &imageFullPath)
 {
 	QTransform trans;
-	Exiv2::Image::AutoPtr exifImage;
+	int orientation = mdCache->getImageOrientation(imageFullPath);
 
-	try
-	{
-		exifImage = Exiv2::ImageFactory::open(imageFullPath.toStdString());
-	}
-	catch (Exiv2::Error &error)
-	{
-		return;
-	}
-
-	exifImage->readMetadata();
-	Exiv2::ExifData &exifData = exifImage->exifData();
-	long orientation = 1;
-
-	if (!exifData.empty())
-	{
-		try
-		{
-			orientation = exifData["Exif.Image.Orientation"].value().toLong();
-		}
-		catch (Exiv2::Error &error)
-		{
-			return;
-		}
-	}
-	
-	switch(orientation)
-	{
+	switch(orientation) {
 		case 2:
 			image = image.mirrored(true, false);
 			break;
@@ -310,53 +303,63 @@ void ImageView::rotateByExifRotation(QImage &image, const QString &imageFullPath
 
 void ImageView::transform()
 {
-	if (GData::exifRotationEnabled)
-	{
+	if (GData::exifRotationEnabled)	{
 		rotateByExifRotation(displayImage, currentImageFullPath);
 	}
 
-	if (GData::rotation)
-	{
+	if (GData::rotation) {
 		QTransform trans;
 		trans.rotate(GData::rotation);
 		displayImage = displayImage.transformed(trans, Qt::SmoothTransformation);
 	}
 
-	if (GData::flipH || GData::flipV)
-	{
+	if (GData::flipH || GData::flipV) {
 		displayImage = displayImage.mirrored(GData::flipH, GData::flipV);
 	}
 
-	preCroppedWidth = displayImage.width();
-	preCroppedHeight = displayImage.height();
-	if (GData::cropLeft || GData::cropTop || GData::cropWidth || GData::cropHeight)
-	{
-		displayImage = displayImage.copy(	
-								GData::cropLeft,
-								GData::cropTop,
-								displayImage.width() - GData::cropWidth - GData::cropLeft,
-								displayImage.height() - GData::cropHeight - GData::cropTop);
+	int cropLeftPercentPixels = 0, cropTopPercentPixels = 0, cropWidthPercentPixels = 0, cropHeightPercentPixels = 0;
+	bool croppingOn = false;
+	if (GData::cropLeftPercent || GData::cropTopPercent
+			|| GData::cropWidthPercent || GData::cropHeightPercent)	{
+		croppingOn = true;
+		cropLeftPercentPixels = (displayImage.width() * GData::cropLeftPercent) / 100;
+		cropTopPercentPixels = (displayImage.height() * GData::cropTopPercent) / 100;
+		cropWidthPercentPixels = (displayImage.width() * GData::cropWidthPercent) / 100;
+		cropHeightPercentPixels = (displayImage.height() * GData::cropHeightPercent) / 100;
+	}
+	
+	if (GData::cropLeft || GData::cropTop || GData::cropWidth || GData::cropHeight)	{
+		displayImage = displayImage.copy(
+			GData::cropLeft + cropLeftPercentPixels,
+			GData::cropTop + cropTopPercentPixels,
+			displayImage.width() - GData::cropLeft - GData::cropWidth - cropLeftPercentPixels - cropWidthPercentPixels,
+			displayImage.height() - GData::cropTop - GData::cropHeight - cropTopPercentPixels - cropHeightPercentPixels);
+	} else {
+		if (croppingOn)	{
+			displayImage = displayImage.copy(
+					cropLeftPercentPixels,
+					cropTopPercentPixels,
+					displayImage.width() - cropLeftPercentPixels - cropWidthPercentPixels,
+					displayImage.height() - cropTopPercentPixels - cropHeightPercentPixels);
+		}
 	}
 }
 
 void ImageView::mirror()
 {
-	switch(mirrorLayout)
-	{
-		case LayDual:
-		{
+	switch(mirrorLayout) {
+		case LayDual: {
 			mirrorImage = QImage(displayImage.width() * 2, displayImage.height(),
-																		QImage::QImage::Format_ARGB32);
+													QImage::QImage::Format_ARGB32);
 			QPainter painter(&mirrorImage);
 			painter.drawImage(0, 0, displayImage);
 			painter.drawImage(displayImage.width(), 0, displayImage.mirrored(true, false));
 			break;
 		}
 
-		case LayTriple:
-		{
+		case LayTriple: {
 			mirrorImage = QImage(displayImage.width() * 3, displayImage.height(),
-																		QImage::QImage::Format_ARGB32);
+													QImage::QImage::Format_ARGB32);
 			QPainter painter(&mirrorImage);
 			painter.drawImage(0, 0, displayImage);
 			painter.drawImage(displayImage.width(), 0, displayImage.mirrored(true, false));
@@ -364,10 +367,9 @@ void ImageView::mirror()
 			break;
 		}
 
-		case LayQuad:
-		{
+		case LayQuad: {
 			mirrorImage = QImage(displayImage.width() * 2, displayImage.height() * 2,
-																		QImage::QImage::Format_ARGB32);
+													QImage::QImage::Format_ARGB32);
 			QPainter painter(&mirrorImage);
 			painter.drawImage(0, 0, displayImage);
 			painter.drawImage(displayImage.width(), 0, displayImage.mirrored(true, false));
@@ -377,10 +379,9 @@ void ImageView::mirror()
 			break;
 		}
 
-		case LayVDual:
-		{
+		case LayVDual: {
 			mirrorImage = QImage(displayImage.width(), displayImage.height() * 2,
-																		QImage::QImage::Format_ARGB32);
+													QImage::QImage::Format_ARGB32);
 			QPainter painter(&mirrorImage);
 			painter.drawImage(0, 0, displayImage);
 			painter.drawImage(0, displayImage.height(), displayImage.mirrored(false, true));
@@ -391,7 +392,7 @@ void ImageView::mirror()
 	displayImage = mirrorImage;
 }
 
-static inline int bound0_255(int val)
+static inline int bound0To255(int val)
 {
 	return ((val > 255)? 255 : (val < 0)? 0 : val);
 }
@@ -400,19 +401,21 @@ static inline int hslValue(double n1, double n2, double hue)
 {
 	double value;
 
-	if (hue > 255)
+	if (hue > 255) {
 		hue -= 255;
-	else if (hue < 0)
+	} else if (hue < 0) {
 		hue += 255;
+	}
 
-	if (hue < 42.5)
+	if (hue < 42.5) {
 		value = n1 + (n2 - n1) * (hue / 42.5);
-	else if (hue < 127.5)
+	} else if (hue < 127.5) {
 		value = n2;
-	else if (hue < 170)
+	} else if (hue < 170) {
 		value = n1 + (n2 - n1) * ((170 - hue) / 42.5);
-	else
+	} else {
 		value = n1;
+	}
 
 	return ROUND(value * 255.0);
 }
@@ -423,46 +426,42 @@ void rgbToHsl(int r, int g, int b, unsigned char *hue, unsigned char *sat, unsig
 	int		min, max;
 	int		delta;
 
-	if (r > g)	
-	{
+	if (r > g) {
 		max = MAX(r, b);
 		min = MIN(g, b);
-	}
-	else
-	{
+	} else {
 		max = MAX(g, b);
 		min = MIN(r, b);
 	}
 
 	l = (max + min) / 2.0;
 
-	if (max == min)
-	{
+	if (max == min) {
 		s = 0.0;
 		h = 0.0;
-	}
-	else
-	{
+	} else {
 		delta = (max - min);
 
-		if (l < 128)
+		if (l < 128) {
 			s = 255 * (double) delta / (double) (max + min);
-		else
+		} else {
 			s = 255 * (double) delta / (double) (511 - max - min);
+		}
 
-		if (r == max)
+		if (r == max) {
 			h = (g - b) / (double) delta;
-		else if (g == max)
+		} else if (g == max) {
 			h = 2 + (b - r) / (double) delta;
-		else
+		} else {
 			h = 4 + (r - g) / (double) delta;
+		}
 
 		h = h * 42.5;
-
-		if (h < 0)
+		if (h < 0) {
 			h += 255;
-		else if (h > 255)
+		} else if (h > 255) {
 			h -= 255;
+		}
 	}
 
 	*hue = ROUND(h);
@@ -473,15 +472,12 @@ void rgbToHsl(int r, int g, int b, unsigned char *hue, unsigned char *sat, unsig
 void hslToRgb(double h, double s, double l,
 					unsigned char *red, unsigned char *green, unsigned char *blue)
 {
-	if (s == 0)
-	{
+	if (s == 0) {
 		/* achromatic case */
 		*red = l;
 		*green = l;
 		*blue = l;
-	}
-	else
-	{
+	} else {
 		double m1, m2;
 
 		if (l < 128)
@@ -507,91 +503,92 @@ void ImageView::colorize()
 	unsigned char h, s, l;
 	static unsigned char contrastTransform[256];	
 	static unsigned char brightTransform[256];
+	bool hasAlpha = displayImage.hasAlphaChannel();
 
-	if (displayImage.format() == QImage::Format_Indexed8)
+	if (displayImage.colorCount()) {
 		displayImage = displayImage.convertToFormat(QImage::Format_RGB32);
+	}
 
-	if (GData::brightContrastEnabled)
-	{
-		int i;
-		float contrast = ((float)GData::contrastVal / 100.0);
-		float brightness = ((float)GData::brightVal / 100.0);
-		
-		for(i = 0; i < 256; ++i)
-		{
-			if (i < (int)(128.0f + 128.0f * tan(contrast)) && i > (int)(128.0f - 128.0f * tan(contrast)))
-				contrastTransform[i] = (i - 128) / tan(contrast) + 128;
-			else if (i >= (int)(128.0f + 128.0f * tan(contrast)))
-				contrastTransform[i] = 255;
-			else
-				contrastTransform[i] = 0;
-		}
-
-		for (i = 0; i < 256; ++i)
-		{
-			brightTransform[i] = MIN(255,(int)((255.0 * pow(i / 255.0, 1.0 / brightness)) + 0.5));
+	int i;
+	float contrast = ((float)GData::contrastVal / 100.0);
+	float brightness = ((float)GData::brightVal / 100.0);
+	
+	for(i = 0; i < 256; ++i) {
+		if (i < (int)(128.0f + 128.0f * tan(contrast)) && i > (int)(128.0f - 128.0f * tan(contrast))) {
+			contrastTransform[i] = (i - 128) / tan(contrast) + 128;
+		} else if (i >= (int)(128.0f + 128.0f * tan(contrast))) {
+			contrastTransform[i] = 255;
+		} else {
+			contrastTransform[i] = 0;
 		}
 	}
 
-	for(y = 0; y < displayImage.height(); ++y)
-	{
+	for (i = 0; i < 256; ++i) {
+		brightTransform[i] = MIN(255,(int)((255.0 * pow(i / 255.0, 1.0 / brightness)) + 0.5));
+	}
+
+	for(y = 0; y < displayImage.height(); ++y) {
+
 		line = (QRgb *)displayImage.scanLine(y);
- 
-		for(x = 0; x < displayImage.width(); ++x)
-		{
-			r = qRed(line[x]);
-			g = qGreen(line[x]);
-			b = qBlue(line[x]);
+		for(x = 0; x < displayImage.width(); ++x) {
+			r = GData::rNegateEnabled? bound0To255(255 - qRed(line[x])) : qRed(line[x]);
+			g = GData::gNegateEnabled? bound0To255(255 - qGreen(line[x])) : qGreen(line[x]);
+			b = GData::bNegateEnabled? bound0To255(255 - qBlue(line[x])) : qBlue(line[x]);
 
-			if (GData::brightContrastEnabled)
-			{
-				r = bound0_255(contrastTransform[r]);
-				g = bound0_255(contrastTransform[g]);
-				b = bound0_255(contrastTransform[b]);
-				r = bound0_255(brightTransform[r]);
-				g = bound0_255(brightTransform[g]);
-				b = bound0_255(brightTransform[b]);
+			r = bound0To255((r * (GData::redVal + 100)) / 100);
+			g = bound0To255((g * (GData::greenVal + 100)) / 100);
+			b = bound0To255((b * (GData::blueVal + 100)) / 100);
+
+			r = bound0To255(brightTransform[r]);
+			g = bound0To255(brightTransform[g]);
+			b = bound0To255(brightTransform[b]);
+
+			r = bound0To255(contrastTransform[r]);
+			g = bound0To255(contrastTransform[g]);
+			b = bound0To255(contrastTransform[b]);
+
+			rgbToHsl(r, g, b, &h, &s, &l);
+			h = GData::colorizeEnabled? GData::hueVal : h + GData::hueVal;
+			s = bound0To255(((s * GData::saturationVal) / 100));
+			l = bound0To255(((l * GData::lightnessVal) / 100));
+			hslToRgb(h, s, l, &hr, &hg, &hb);
+
+			r = GData::hueRedChannel? hr : qRed(line[x]);
+			g = GData::hueGreenChannel? hg : qGreen(line[x]);
+			b = GData::hueBlueChannel? hb: qBlue(line[x]);
+
+			if (hasAlpha) {
+				line[x] = qRgba(r, g, b, qAlpha(line[x]));
+			} else {
+				line[x] = qRgb(r, g, b);
 			}
-
-			if (GData::hueSatEnabled)
-			{
-				rgbToHsl(r, g, b, &h, &s, &l);
-									
-				if (GData::colorizeEnabled)
-					h = GData::hueVal;
-				else
-					h += GData::hueVal;
-
-				s = bound0_255(((s * GData::saturationVal) / 100));
-				l = bound0_255(((l * GData::lightnessVal) / 100));
-
-				hslToRgb(h, s, l, &hr, &hg, &hb);
-
-				r = GData::hueRedChannel? hr : qRed(line[x]);
-				g = GData::hueGreenChannel? hg : qGreen(line[x]);
-				b = GData::hueBlueChannel? hb: qBlue(line[x]);
-			}
-
-			line[x] = qRgb(r, g, b);
 		}
 	}
 }
 
 void ImageView::refresh()
 {
-	if (isAnimation)
+	if (isAnimation) {
 		return;
+	}
 
-	if (GData::scaledWidth)
+	if (GData::scaledWidth) {
 		displayImage = origImage.scaled(GData::scaledWidth, GData::scaledHeight,
 											Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-	else
+	} else {
 		displayImage = origImage;
+	}
+
 	transform();
-	if (GData::brightContrastEnabled || GData::hueSatEnabled)
+
+	if (GData::colorsActive || GData::keepTransform) {
 		colorize();
-	if (mirrorLayout)
+	}
+
+	if (mirrorLayout) {
 		mirror();
+	}
+
 	displayPixmap = QPixmap::fromImage(displayImage);
 	imageLabel->setPixmap(displayPixmap);
 	resizeImage();
@@ -599,27 +596,26 @@ void ImageView::refresh()
 
 void ImageView::reload()
 {
+	isAnimation = false;
 	if (GData::enableImageInfoFS) {
-		if (currentImageFullPath.isEmpty())
+		if (currentImageFullPath.left(1) == ":") {
+			setInfo("No Image");
+		} else if (currentImageFullPath.isEmpty()) { 
 			setInfo("Clipboard");
-		else
+		} else {
 			setInfo(QFileInfo(currentImageFullPath).fileName());
+		}
 	}
 
 	if (!GData::keepTransform) {
-		GData::cropLeft = 0;
-		GData::cropTop = 0;
-		GData::cropWidth = 0;
-		GData::cropHeight = 0;
+		GData::cropLeftPercent = GData::cropTopPercent = GData::cropWidthPercent = GData::cropHeightPercent = 0;
 		GData::rotation = 0;
-		GData::flipH = false;
-		GData::flipV = false;
+		GData::flipH = GData::flipV = false;
 	}
-	GData::scaledWidth = 0;
-	GData::scaledHeight = 0;
+	GData::scaledWidth = GData::scaledHeight = 0;
+	GData::cropLeft = GData::cropTop = GData::cropWidth = GData::cropHeight = 0;
 
-	if (newImage || currentImageFullPath.isEmpty())
-	{
+	if (newImage || currentImageFullPath.isEmpty())	{
 		newImage = true;
 		currentImageFullPath = CLIPBOARD_IMAGE_NAME;
 		origImage.load(":/images/no_image.png");
@@ -628,37 +624,39 @@ void ImageView::reload()
 		imageLabel->setPixmap(displayPixmap);
 		pasteImage();
 		mainWindow->setWindowTitle(tr("Clipboard") + " - Phototonic");
-		isAnimation = false;
 		return;
 	}
 
 	imageReader.setFileName(currentImageFullPath);
-
-	isAnimation = GData::enableAnimations? imageReader.supportsAnimation() : false;
-	if (isAnimation)
-	{
-		if (anim)
+	if (GData::enableAnimations && imageReader.supportsAnimation()) {
+		if (anim) {
 			delete anim;
+		}
 		anim = new QMovie(currentImageFullPath);
-		imageLabel->setMovie(anim);
-		anim->start();
+
+		if (anim->frameCount() > 1) {
+			isAnimation = true;
+			imageLabel->setMovie(anim);
+			anim->start();
+		}
 	}
-	else
-	{
-		if (imageReader.size().isValid())
-		{
-			origImage.load(currentImageFullPath);
+
+	if (!isAnimation) {
+		if (imageReader.size().isValid() && imageReader.read(&origImage)) {
 			displayImage = origImage;
 			transform();
-			if (GData::brightContrastEnabled || GData::hueSatEnabled)
+			if (GData::colorsActive || GData::keepTransform) {
 				colorize();
-			if (mirrorLayout)
+			}
+			if (mirrorLayout) {
 				mirror();
+			}
 			displayPixmap = QPixmap::fromImage(displayImage);
-		}
-		else
+		} else {
 			displayPixmap = QIcon::fromTheme("image-missing", 
-													QIcon(":/images/error_image.png")).pixmap(128, 128);
+										QIcon(":/images/error_image.png")).pixmap(128, 128);
+			setInfo(imageReader.errorString());
+		}
 
 		imageLabel->setPixmap(displayPixmap);
 	}
@@ -696,9 +694,11 @@ void ImageView::loadImage(QString imageFileName)
 	tempDisableResize = false;
 	currentImageFullPath = imageFileName;
 
-	if (!GData::keepZoomFactor)
+	if (!GData::keepZoomFactor) {
 		GData::imageZoomFactor = 1.0;
+	}
 
+	QApplication::processEvents();
 	reload();
 }
 
@@ -748,6 +748,19 @@ void ImageView::mouseDoubleClickEvent(QMouseEvent *event)
 void ImageView::mousePressEvent(QMouseEvent *event)
 {
 	if (event->button() == Qt::LeftButton) {
+
+		if (event->modifiers() == Qt::ControlModifier) {
+			cropOrigin = event->pos();
+		    if (!cropBand)
+		        cropBand = new CropRubberBand(this);
+	        cropBand->show();
+			cropBand->setGeometry(QRect(cropOrigin , event->pos()).normalized());
+		} else {
+			if (cropBand) {
+				cropBand->hide();
+			}
+		}
+
 		setMouseMoveData(true, event->x(), event->y());
 		QApplication::setOverrideCursor(Qt::ClosedHandCursor);
 		event->accept();
@@ -757,11 +770,62 @@ void ImageView::mousePressEvent(QMouseEvent *event)
 
 void ImageView::mouseReleaseEvent(QMouseEvent *event)
 {
-	if (event->button() == Qt::LeftButton)
-	{
+	if (event->button() == Qt::LeftButton) {
 		setMouseMoveData(false, 0, 0);
-		while (QApplication::overrideCursor())
+		while (QApplication::overrideCursor()) {
 			QApplication::restoreOverrideCursor();
+		}
+
+		if (cropBand && cropBand->isVisible()) {
+			setFeedback(tr("Selection size: ")
+			+ QString::number(cropBand->width())
+			+ "x"
+			+ QString::number(cropBand->height()));
+		}
+	}
+
+	QWidget::mouseReleaseEvent(event);
+}
+
+void ImageView::cropToSelection()
+{
+	if (cropBand && cropBand->isVisible()) {
+
+		QPoint bandTL = mapToGlobal(cropBand->geometry().topLeft());
+		QPoint bandBR = mapToGlobal(cropBand->geometry().bottomRight());
+
+		bandTL = imageLabel->mapFromGlobal(bandTL);
+		bandBR = imageLabel->mapFromGlobal(bandBR);
+		
+		double scaledX = imageLabel->rect().width();
+		double scaledY = imageLabel->rect().height();
+		scaledX = displayPixmap.width() / scaledX;
+		scaledY = displayPixmap.height() / scaledY;
+
+		bandTL.setX(int(bandTL.x() * scaledX));
+		bandTL.setY(int(bandTL.y() * scaledY));
+		bandBR.setX(int(bandBR.x() * scaledX));
+		bandBR.setY(int(bandBR.y() * scaledY));
+
+		int cropLeft = bandTL.x();
+		int cropTop = bandTL.y();
+		int cropWidth = displayPixmap.width() - bandBR.x();
+		int cropHeight = displayPixmap.height() -  bandBR.y();
+
+		if (cropLeft > 0)
+			GData::cropLeft += cropLeft;
+		if (cropTop > 0)
+			GData::cropTop += cropTop;
+		if (cropWidth > 0)
+			GData::cropWidth += cropWidth;
+		if (cropHeight > 0)
+			GData::cropHeight += cropHeight;
+
+		cropBand->hide();
+		refresh();
+	} else {
+		QMessageBox msgBox;
+		msgBox.warning(this, tr("No selection"), tr("Hold down the Ctrl key and select a region using the mouse."));
 	}
 }
 
@@ -776,36 +840,40 @@ void ImageView::setMouseMoveData(bool lockMove, int lMouseX, int lMouseY)
 
 void ImageView::mouseMoveEvent(QMouseEvent *event)
 {
-	if (moveImageLocked) 
-	{
-		int newX = layoutX + (event->pos().x() - mouseX);
-		int newY = layoutY + (event->pos().y() - mouseY);
-		bool needToMove = false;
-
-		if (imageLabel->size().width() > size().width())
-		{
-			if (newX > 0)
-				newX = 0;
-			else if (newX < (size().width() - imageLabel->size().width()))
-				newX = (size().width() - imageLabel->size().width());
-			needToMove = true;
+	if (event->modifiers() == Qt::ControlModifier) {
+		if (cropBand && cropBand->isVisible()) {
+			cropBand->setGeometry(QRect(cropOrigin , event->pos()).normalized());
 		}
-		else
-			newX = layoutX;
-
-		if (imageLabel->size().height() > size().height())
+	} else {
+		if (moveImageLocked) 
 		{
-			if (newY > 0)
-				newY = 0;
-			else if (newY < (size().height() - imageLabel->size().height()))
-				newY = (size().height() - imageLabel->size().height());
-			needToMove = true;
-		}
-		else
-			newY = layoutY;
+			int newX = layoutX + (event->pos().x() - mouseX);
+			int newY = layoutY + (event->pos().y() - mouseY);
+			bool needToMove = false;
 
-		if (needToMove)
-			imageLabel->move(newX, newY);
+			if (imageLabel->size().width() > size().width()) 	{
+				if (newX > 0)
+					newX = 0;
+				else if (newX < (size().width() - imageLabel->size().width()))
+					newX = (size().width() - imageLabel->size().width());
+				needToMove = true;
+			}
+			else
+				newX = layoutX;
+
+			if (imageLabel->size().height() > size().height()) {
+				if (newY > 0)
+					newY = 0;
+				else if (newY < (size().height() - imageLabel->size().height()))
+					newY = (size().height() - imageLabel->size().height());
+				needToMove = true;
+			}
+			else
+				newY = layoutY;
+
+			if (needToMove)
+				imageLabel->move(newX, newY);
+		}
 	}
 }
 
@@ -815,8 +883,7 @@ void ImageView::keyMoveEvent(int direction)
 	int newY = layoutY = imageLabel->pos().y();
 	bool needToMove = false;
 
-	switch (direction)
-	{
+	switch (direction) {
 		case MoveLeft:
 			newX += 50;
 			break;
@@ -831,34 +898,32 @@ void ImageView::keyMoveEvent(int direction)
 			break;
 	}
 
-	if (imageLabel->size().width() > size().width())
-	{
-		if (newX > 0)
+	if (imageLabel->size().width() > size().width()) {
+		if (newX > 0) {
 			newX = 0;
-		else if (newX < (size().width() - imageLabel->size().width()))
+		} else if (newX < (size().width() - imageLabel->size().width())) {
 			newX = (size().width() - imageLabel->size().width());
+		}			
 		needToMove = true;
-	}
-	else
+	} else {
 		newX = layoutX;
-
-	if (imageLabel->size().height() > size().height())
-	{
-		if (newY > 0)
-			newY = 0;
-		else if (newY < (size().height() - imageLabel->size().height()))
-			newY = (size().height() - imageLabel->size().height());
-		needToMove = true;
 	}
-	else
-		newY = layoutY;
 
-	if (needToMove)
-	{
+	if (imageLabel->size().height() > size().height()) {
+		if (newY > 0) {
+			newY = 0;
+		} else if (newY < (size().height() - imageLabel->size().height())) {
+			newY = (size().height() - imageLabel->size().height());
+		}
+		needToMove = true;
+	} else {
+		newY = layoutY;
+	}
+
+	if (needToMove) {
 		int i;
 
-		switch (direction)
-		{
+		switch (direction) {
 			case MoveLeft:
 				for (i = imageLabel->pos().x(); i <= newX; ++i)
 					imageLabel->move(newX, newY);
@@ -907,7 +972,7 @@ void ImageView::saveImage()
 																			GData::defaultSaveQuality))
 	{
 		QMessageBox msgBox;
-		msgBox.critical(this, tr("Error"), tr("Failed to save image"));
+		msgBox.critical(this, tr("Error"), tr("Failed to save image."));
 		return;
 	}
 
@@ -920,12 +985,12 @@ void ImageView::saveImage()
 		catch (Exiv2::Error &error)
 		{
 			QMessageBox msgBox;
-			msgBox.critical(this, tr("Error"), tr("Failed to save Exif metadata"));
+			msgBox.critical(this, tr("Error"), tr("Failed to save Exif metadata."));
 		}
 	}
 
 	reload();
-	setFeedback(tr("Image saved"));
+	setFeedback(tr("Image saved."));
 }
 
 void ImageView::saveImageAs()
@@ -934,48 +999,44 @@ void ImageView::saveImageAs()
 	Exiv2::Image::AutoPtr newExifImage;
 	bool exifError = false;
 
+	setCursorHiding(false);
+
 	QString fileName = QFileDialog::getSaveFileName(this,
 		tr("Save image as"),
 		currentImageFullPath,
-		tr("Image Files") + " (*.jpg *.jpeg *.jpe *.png *.bmp *.tiff *.tif *.ppm *.xbm *.xpm)");
+		tr("Images") + " (*.jpg *.jpeg *.jpe *.png *.bmp *.ppm *.pgm *.pbm *.xbm *.xpm)");
 		
-	if (!fileName.isEmpty())
-	{
-		try
-		{
+	if (!fileName.isEmpty()) {
+		try {
 			exifImage = Exiv2::ImageFactory::open(currentImageFullPath.toStdString());
 			exifImage->readMetadata();
 		}
-		catch (Exiv2::Error &error)
-		{
+		catch (Exiv2::Error &error)	{
 			exifError = true;
 		}
 
 	
-		if (!displayPixmap.save(fileName, 0, GData::defaultSaveQuality))
-		{
+		if (!displayPixmap.save(fileName, 0, GData::defaultSaveQuality)) {
 			QMessageBox msgBox;
-			msgBox.critical(this, tr("Error"), tr("Failed to save image"));
-		}
-		else
-		{
-			if (!exifError)
-			{
-				try
-				{
+			msgBox.critical(this, tr("Error"), tr("Failed to save image."));
+		} else {
+			if (!exifError) {
+				try	{
 					newExifImage = Exiv2::ImageFactory::open(fileName.toStdString());
 					newExifImage->setMetadata(*exifImage);
 					newExifImage->writeMetadata();
 				}
-				catch (Exiv2::Error &error)
-				{
+				catch (Exiv2::Error &error) {
 					exifError = true;
 				}
 			}
 		
-			setFeedback(tr("Image saved"));
+			setFeedback(tr("Image saved."));
 		}
 	}
+	if (mainWindow->isFullScreen()) {
+		setCursorHiding(true);
+	}
 }
 
 void ImageView::contextMenuEvent(QContextMenuEvent *)
@@ -988,12 +1049,12 @@ void ImageView::contextMenuEvent(QContextMenuEvent *)
 
 int ImageView::getImageWidthPreCropped()
 {
-	return preCroppedWidth;
+	return origImage.width();
 }
 
 int ImageView::getImageHeightPreCropped()
 {
-	return preCroppedHeight;
+	return origImage.height();
 }
 
 bool ImageView::isNewImage()
@@ -1011,8 +1072,7 @@ void ImageView::pasteImage()
 	if (isAnimation)
 		return;
 
-	if (!QApplication::clipboard()->image().isNull())
-	{
+	if (!QApplication::clipboard()->image().isNull()) {
 		origImage = QApplication::clipboard()->image();
 		refresh();
 	}
