@@ -19,13 +19,7 @@
 #ifndef INFOVIEW_H
 #define INFOVIEW_H
 
-#include <QTableView>
-#include <QStandardItem>
-#include <QMenu>
-#include <QHeaderView>
-#include <QDebug>
-#include <QApplication>
-#include <QClipboard>
+#include <QtWidgets>
 
 class InfoView : public QTableView
 {
@@ -35,6 +29,7 @@ public:
 	InfoView(QWidget *parent);
 	void clear();
 	void addEntry(QString &key, QString &value);
+	void addTitleEntry(QString title);
 
 public slots:
 	void showInfoViewMenu(QPoint pt);
